#!/usr/bin/python
#
# Copyright 2008 IBM Corp.
#
# Authors:
#    Deepti B. Kalakeri <dkalaker@in.ibm.com>
#    Kaitlin Rupert <karupert@us.ibm.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#

# This test case is used to verify the Xen_VirtualSystemSettingDataComponent
# association.
#
# Ex: Command and the fields that are verified are given below.
# wbemcli ain -ac Xen_VirtualSystemSettingDataComponent \
# 'http://localhost:5988/root/virt:Xen_VirtualSystemSettingData.\
#  InstanceID="Xen:domgst"'
#
# Output:
# localhost:5988/root/virt:Xen_ProcResourceAllocationSettingData.\
# InstanceID="domgst/0" 
# localhost:5988/root/virt:Xen_NetResourceAllocationSettingData\
# .InstanceID="domgst/00:22:33:aa:bb:cc" 
# localhost:5988/root/virt:Xen_DiskResourceAllocationSettingData.\
# InstanceID="domgst/xvda"
# localhost:5988/root/virt:Xen_MemResourceAllocationSettingData.\
# InstanceID="domgst/mem"
# 
# 
# 
#                                               Date : 01-01-2008


import sys
from XenKvmLib import enumclass
from VirtLib import utils
from XenKvmLib import assoc
from XenKvmLib.test_doms import destroy_and_undefine_all 
from XenKvmLib import vxml
from XenKvmLib.classes import get_typed_class
from CimTest.Globals import logger, CIM_ERROR_ASSOCIATORS
from XenKvmLib.const import do_main
from CimTest.ReturnCodes import PASS, FAIL, XFAIL_RC

bug_libvirt = "00009"
sup_types = ['Xen', 'XenFV', 'KVM', 'LXC']

test_dom    = "VSSDC_dom"
test_vcpus  = 1
test_mem    = 128
test_mac    = "00:11:22:33:44:aa"

def check_rasd_values(id, exp_id):
    try:
        if id != exp_id:
            logger.error("Returned %s instead of %s", id, exp_id)
            return FAIL
 
    except Exception, detail :
        logger.error("Exception evaluating InstanceID: %s", detail)
        return FAIL

    return PASS

def assoc_values(ip, assoc_info, virt="Xen"):
    """
        The association info of 
        Xen_VirtualSystemSettingDataComponent is
        verified. 
    """
    status = PASS
    if virt == 'LXC':
        input_device = "mouse:usb"
    elif virt == 'Xen':
        input_device = "mouse:xen"
    else:
        input_device = "mouse:ps2"

    rasd_list = {
                 "proc_rasd" : '%s/%s' %(test_dom, "proc"), 
                 "net_rasd"  : '%s/%s' %(test_dom,test_mac),
                 "disk_rasd" : '%s/%s' %(test_dom, test_disk),
                 "mem_rasd"  : '%s/%s' %(test_dom, "mem"),
                 "input_rasd": '%s/%s' %(test_dom, input_device),
                 "grap_rasd" : '%s/%s' %(test_dom, "graphics")

                }

    try: 
        if len(assoc_info) <= 0: 
            logger.error("No RASD instances returned")
            return FAIL

        proc_cn = get_typed_class(virt, 'ProcResourceAllocationSettingData')
        net_cn = get_typed_class(virt, 'NetResourceAllocationSettingData')
        disk_cn = get_typed_class(virt, 'DiskResourceAllocationSettingData')
        mem_cn = get_typed_class(virt, 'MemResourceAllocationSettingData')
        input_cn = get_typed_class(virt, 'InputResourceAllocationSettingData')
        grap_cn = get_typed_class(virt, 'GraphicsResourceAllocationSettingData')
    
        for inst in assoc_info: 
            if inst.classname == proc_cn:
                status = check_rasd_values(inst['InstanceID'], 
                                           rasd_list['proc_rasd'])
            elif inst.classname == net_cn:
                status = check_rasd_values(inst['InstanceID'], 
                                           rasd_list['net_rasd'])
            elif inst.classname == disk_cn: 
                status = check_rasd_values(inst['InstanceID'], 
                                           rasd_list['disk_rasd'])
            elif inst.classname == mem_cn: 
                status = check_rasd_values(inst['InstanceID'], 
                                           rasd_list['mem_rasd'])
            elif inst.classname == input_cn:
                status = check_rasd_values(inst['InstanceID'],
                                           rasd_list['input_rasd'])
                if status == FAIL and virt == "LXC":
                    return XFAIL_RC(bug_libvirt)
            elif inst.classname == grap_cn:
                status = check_rasd_values(inst['InstanceID'],
                                           rasd_list['grap_rasd'])
            else:
                logger.error("Unexpected RASD instance type" )
                status = FAIL

            if status == FAIL:
                logger.error("Mistmatching association value" )
                break  

    except  Exception, detail :
        logger.error("Exception in assoc_values function: %s", detail)
        status = FAIL

    return status

@do_main(sup_types)
def main():
    options = main.options
    status = PASS

    destroy_and_undefine_all(options.ip)

    global test_disk
    if options.virt == "Xen":
        test_disk = "xvdb"
    elif options.virt == "LXC":
        test_disk = "/tmp"
    else:
        test_disk = "hdb"

    virt_xml = vxml.get_class(options.virt)
    if options.virt == 'LXC':
        cxml = virt_xml(test_dom)
    else:
        cxml = virt_xml(test_dom, vcpus = test_vcpus, 
                        mac = test_mac, disk = test_disk)

    ret = cxml.cim_define(options.ip)
    if not ret:
        logger.error("Failed to define the dom: %s", test_dom)
        return FAIL

    ret = cxml.cim_start(options.ip)
    if ret != PASS:
        cxml.undefine(options.ip)
        logger.error("Failed to start the dom: %s", test_dom)
        return ret

    if options.virt == "XenFV":
        instIdval = "Xen:%s" % test_dom
    else:
        instIdval = "%s:%s" % (options.virt, test_dom)
    
    try:
        an = get_typed_class(options.virt, 'VirtualSystemSettingDataComponent')
        cn = get_typed_class(options.virt, 'VirtualSystemSettingData')
        assoc_info = assoc.AssociatorNames(options.ip, an, cn,
                                           InstanceID = instIdval)
        status = assoc_values(options.ip, assoc_info, options.virt)

    except  Exception, detail :
        logger.error(CIM_ERROR_ASSOCIATORS, an)
        logger.error("Exception : %s", detail)
        status = FAIL

    cxml.destroy(options.ip)
    cxml.undefine(options.ip)
    return status

if __name__ == "__main__":
    sys.exit(main())
